@@ -2,18 +2,6 @@
 #include "any_missing.h"
 #include "is_integerish.h"
 
-const char * guessType(SEXP x) {
-    SEXP attr = getAttrib(x, R_ClassSymbol);
-    if (!isNull(attr))
-        return CHAR(STRING_ELT(attr, 0));
-
-    attr = getAttrib(x, R_DimSymbol);
-    if (!isNull(attr) && isVectorAtomic(x))
-        return length(attr) == 2 ? "matrix" : "array";
-
-    return type2char(TYPEOF(x));
-}
-
 Rboolean isStrictlyNumeric(SEXP x) {
     switch(TYPEOF(x)) {
         case REALSXP: return TRUE;
@@ -36,7 +24,6 @@ Rboolean isRList(SEXP x) {
     return FALSE;
 }
 
-
 /* ncols and nrows is bugged for data frames:
  * (a) data.frames are treated like lists and thus you get length() back
  * (b) reports wrong dimension for zero-column data frames
