@@ -6,7 +6,7 @@
 #include <Rinternals.h>
 
 SEXP c_qassert(SEXP, SEXP, SEXP);
-SEXP c_qtest(SEXP, SEXP, SEXP);
+SEXP c_qtest(SEXP, SEXP, SEXP, SEXP);
 SEXP qassert(SEXP, const char *, const char *);
 Rboolean qtest(SEXP, const char *);
 
